#!/usr/bin/env python

# eq_sat_mask_h_m_s.py

# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
# This program creates 10 equally spaced Saturation Masks.
# The masks are created by using 10 spline curves, with different amounts of overlap for H, M & S.
# 
#
# 
# ------------
#| Change Log |
# ------------
# Version   1 22/08/2022
# Revision  1 
#
# import Image
from gimpfu import *
def eq_sat_mask_h_m_s(image, layer, transition):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()

	frozen = pdb.gimp_image_undo_freeze(image)																# disable undo to increase speed and reduce memory requirements
		
	if not pdb.gimp_selection_is_empty(image):																# check if there is a selection. Note: double negative!
		old_select = pdb.gimp_selection_save(image)															# save pre-existing selection to channel	
	
	pdb.gimp_selection_none(image)																			# remove any pre-existing selection
	
	############# create saturation later #######################
	
	layerNew = pdb.gimp_layer_new_from_drawable(layer, image)												# layerNew is existing name in loop
	pdb.gimp_image_insert_layer(image, layerNew, None, 0)
	
	width   = image.width
	height  = image.height
	type    = RGB_IMAGE
	opacity = 100
	mode    = NORMAL_MODE
	
	layer_copy = pdb.gimp_layer_new(image, width, height, type, "layer_copy", opacity, mode)				# new empty layer
	image.add_layer(layer_copy, 0)																			# layer_copy is existing name in loop
	
	pdb.gimp_image_remove_layer(image, layer)																# no longer needed - just so I could
																											# re-use loop routine with orig. names
																											
	decomposed_image, not_used1, not_used2, not_used3 = pdb.plug_in_decompose(image, layerNew, "HSV", 1)	# decompose image HSV mode
	
	display = pdb.gimp_display_new(decomposed_image)
	
	saturation_layer = decomposed_image.layers[1]															# get saturation layer from HSV

	pdb.gimp_selection_all(decomposed_image)
	pdb.gimp_edit_copy(saturation_layer)																	# copy to buffer to use later
	
	pdb.gimp_display_delete(display)	
	
	floating_sel = pdb.gimp_edit_paste(layer_copy, TRUE)													# copy saturation to layer_copy
		
	mask = pdb.gimp_layer_create_mask(layerNew, ADD_MASK_WHITE)												# add empty mask before loop
	pdb.gimp_layer_add_mask(layerNew, mask)
	pdb.gimp_floating_sel_anchor(floating_sel)
	
	pdb.gimp_layer_set_edit_mask(layerNew, FALSE) 															# make layer active
	
	############# saturation layer created ######################
	
	layerNum = (0)
	
	pdb.gimp_layer_set_edit_mask(layerNew, FALSE) 															# make layer active
		
	splinePointsH = [[0.00,1.0, 0.10,1.0, 0.20,0.0, 0.30,0.0, 0.40,0.0, 0.50,0.0, 0.60,0.0, 0.70,0.0, 0.80,0.0, 0.90,0.0, 1.00,0.0],	# hard spline curves
				[0.00,0.0, 0.10,1.0, 0.20,1.0, 0.30,0.0, 0.40,0.0, 0.50,0.0, 0.60,0.0, 0.70,0.0, 0.80,0.0, 0.90,0.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,1.0, 0.30,1.0, 0.40,0.0, 0.50,0.0, 0.60,0.0, 0.70,0.0, 0.80,0.0, 0.90,0.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,1.0, 0.40,1.0, 0.50,0.0, 0.60,0.0, 0.70,0.0, 0.80,0.0, 0.90,0.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.0, 0.40,1.0, 0.50,1.0, 0.60,0.0, 0.70,0.0, 0.80,0.0, 0.90,0.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.0, 0.40,0.0, 0.50,1.0, 0.60,1.0, 0.70,0.0, 0.80,0.0, 0.90,0.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.0, 0.40,0.0, 0.50,0.0, 0.60,1.0, 0.70,1.0, 0.80,0.0, 0.90,0.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.0, 0.40,0.0, 0.50,0.0, 0.60,0.0, 0.70,1.0, 0.80,1.0, 0.90,0.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.0, 0.40,0.0, 0.50,0.0, 0.60,0.0, 0.70,0.0, 0.80,1.0, 0.90,1.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.0, 0.40,0.0, 0.50,0.0, 0.60,0.0, 0.70,0.0, 0.80,0.0, 0.90,1.0, 1.00,1.0]]
				
	splinePointsM = [[0.00,1.0, 0.10,1.0, 0.20,0.5, 0.30,0.0, 0.40,0.0, 0.50,0.0, 0.60,0.0, 0.70,0.0, 0.80,0.0, 0.90,0.0, 1.00,0.0],	# medium spline curves
				[0.00,0.5, 0.10,1.0, 0.20,1.0, 0.30,0.5, 0.40,0.0, 0.50,0.0, 0.60,0.0, 0.70,0.0, 0.80,0.0, 0.90,0.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.5, 0.20,1.0, 0.30,1.0, 0.40,0.5, 0.50,0.0, 0.60,0.0, 0.70,0.0, 0.80,0.0, 0.90,0.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.5, 0.30,1.0, 0.40,1.0, 0.50,0.5, 0.60,0.0, 0.70,0.0, 0.80,0.0, 0.90,0.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.5, 0.40,1.0, 0.50,1.0, 0.60,0.5, 0.70,0.0, 0.80,0.0, 0.90,0.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.0, 0.40,0.5, 0.50,1.0, 0.60,1.0, 0.70,0.5, 0.80,0.0, 0.90,0.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.0, 0.40,0.0, 0.50,0.5, 0.60,1.0, 0.70,1.0, 0.80,0.5, 0.90,0.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.0, 0.40,0.0, 0.50,0.0, 0.60,0.5, 0.70,1.0, 0.80,1.0, 0.90,0.5, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.0, 0.40,0.0, 0.50,0.0, 0.60,0.0, 0.70,0.5, 0.80,1.0, 0.90,1.0, 1.00,0.5],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.0, 0.40,0.0, 0.50,0.0, 0.60,0.0, 0.70,0.0, 0.80,0.5, 0.90,1.0, 1.00,1.0]]				
		
	splinePointsS = [[0.00,1.0, 0.10,1.0, 0.20,0.7, 0.30,0.3, 0.40,0.0, 0.50,0.0, 0.60,0.0, 0.70,0.0, 0.80,0.0, 0.90,0.0, 1.00,0.0],	# soft spline curves
				[0.00,0.7, 0.10,1.0, 0.20,1.0, 0.30,0.7, 0.40,0.3, 0.50,0.0, 0.60,0.0, 0.70,0.0, 0.80,0.0, 0.90,0.0, 1.00,0.0],
				[0.00,0.3, 0.10,0.7, 0.20,1.0, 0.30,1.0, 0.40,0.7, 0.50,0.3, 0.60,0.0, 0.70,0.0, 0.80,0.0, 0.90,0.0, 1.00,0.0],								
				[0.00,0.0, 0.10,0.3, 0.20,0.7, 0.30,1.0, 0.40,1.0, 0.50,0.7, 0.60,0.3, 0.70,0.0, 0.80,0.0, 0.90,0.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.3, 0.30,0.7, 0.40,1.0, 0.50,1.0, 0.60,0.7, 0.70,0.3, 0.80,0.0, 0.90,0.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.3, 0.40,0.7, 0.50,1.0, 0.60,1.0, 0.70,0.7, 0.80,0.3, 0.90,0.0, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.0, 0.40,0.3, 0.50,0.7, 0.60,1.0, 0.70,1.0, 0.80,0.7, 0.90,0.3, 1.00,0.0],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.0, 0.40,0.0, 0.50,0.3, 0.60,0.7, 0.70,1.0, 0.80,1.0, 0.90,0.7, 1.00,0.3],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.0, 0.40,0.0, 0.50,0.0, 0.60,0.3, 0.70,0.7, 0.80,1.0, 0.90,1.0, 1.00,0.7],
				[0.00,0.0, 0.10,0.0, 0.20,0.0, 0.30,0.0, 0.40,0.0, 0.50,0.0, 0.60,0.0, 0.70,0.3, 0.80,0.7, 0.90,1.0, 1.00,1.0]]
				
	############# START OF REPEATED SECTION #####################
	
	layerNum=(1)																							# integer index for layerName	
	
		
	if transition == 0:
		splinePoints = splinePointsH																		# hard transition mask creation
		
	elif transition == 1:	
		splinePoints = splinePointsM																		# medium transition mask creation
		
	elif transition == 2:	
		splinePoints = splinePointsS																		# soft transition mask creation
			
	for spline in splinePoints:																				# first row of splinePoints used when layerNum=(0), second row when (1), etc.																				
				
		layer_copy1 = pdb.gimp_layer_new_from_drawable(layer_copy,image)									# make a layer for applying curve
		pdb.gimp_image_insert_layer(image,layer_copy1,None,0)
		
		pdb.gimp_drawable_curves_spline(layer_copy1,HISTOGRAM_VALUE,len(spline),spline)						# cycle through splinePoints list at each iteration
		
		pdb.gimp_selection_all(image)																		# copy to buffer		
		pdb.gimp_edit_copy(layer_copy1)	
		
		pdb.gimp_image_remove_layer(image,layer_copy1)														# remove splined layer									
 		floating_sel=pdb.gimp_edit_paste(mask,TRUE)															# paste into mask										
		pdb.gimp_floating_sel_anchor(floating_sel)

 		pdb.gimp_item_set_visible(layerNew,TRUE)											
 		pdb.gimp_selection_none(image)
			
		layer_add = pdb.gimp_layer_new_from_drawable(layerNew, image)										# new layer from originalimage ready for next time around loop
 		pdb.gimp_image_insert_layer(image, layer_add, None, 0)
 		layerName = "Sat "+str(layerNum)																	# layerName string from layerNum
 		pdb.gimp_item_set_name(layer_add, layerName)


		layerNum+=1
	
	############# END OF REPEATED SECTION #######################
	
	pdb.gimp_image_remove_layer(image,layer_copy)															# remove "splined" layer
		
	pdb.gimp_image_remove_layer(image,layerNew)																# remove original layer with mask- no longer needed
	
	thawed = pdb.gimp_image_undo_thaw(image)																# re-enable undo
		
	pdb.gimp_selection_none(image)
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()

register(
	"python_fu_eq_sat_mask_h_m_s",
	"Create Saturation Masks",
	"Create Saturation Masks",
	"David Marsden",
	"David Marsden",
	"August 2022",
	"<Image>/Python-Fu/Equal Saturation Masks H M S...",             										#Menu path
	"RGB*", 
	[
		(PF_OPTION, "transition", "Transition - Overlap", 2, ("Hard", "Medium", "Soft"))
	],
	[],
	eq_sat_mask_h_m_s)

main()
